import json

from mower.solvers.navigation import NavigationSolver
from mower.utils import config
from mower.utils.graph.utils import SceneGraphSolver
from mower.utils.log import logger
from mower.utils.path import get_path
from mower.utils.scene import Scene

from .auto_fight import AutoFight
from .battle_choose import BattleChooseSolver


class FightSolver(SceneGraphSolver):
    def run(
        self,
        auto_choose: bool = True,
    ):
        work = config.conf.work.works[0]
        work_path = get_path(f"@app/works/{work["stage"]}.json")
        with work_path.open("r", encoding="utf-8") as f:
            work_data = json.load(f)
        opers = work_data["opers"]
        groups = work_data["groups"]
        actions = work_data["actions"]
        self.level_name = work["stage"]
        support_oper = work["support_oper"]
        self.no_potion = False
        success = False

        self.check_opers_skill(opers, groups)
        if support_oper:
            support_oper = self.find_support_oper(support_oper, opers, groups)
        if auto_choose:
            super().run()
            if self.no_potion:
                return success
            if not BattleChooseSolver().run(
                self.level_name,
                opers,
                groups,
                support_oper,
            ):
                return success
        while work["retry_times"] + 1 > 0:
            super().run()
            if self.no_potion:
                break
            if AutoFight().run(self.level_name, opers, actions, groups):
                success = True
                config.conf.work.works.pop(0)
                break
            work["retry_times"] -= 1
        config.save_conf()
        return success

    def check_opers_skill(self, opers: list, groups: list):
        for oper in opers:
            if "skill" not in oper:
                oper["skill"] = 1
            if "skill_usage" not in oper:
                oper["skill_usage"] = 0
        for group in groups:
            for oper in group["opers"]:
                if "skill" not in oper:
                    oper["skill"] = 1
                if "skill_usage" not in oper:
                    oper["skill_usage"] = 0

    def find_support_oper(self, support_oper: str, opers: list, groups: list):
        for op in opers + groups:
            if op["name"] == support_oper:
                op["support"] = True
                return [op]

    def transition(self):
        if (scene := self.scene()) == Scene.OPERATOR_RECOVER_POTION:
            if config.conf.work.use_medicine:
                logger.info("体力不足，使用理智药")
                self.tap((1635, 865))
                return
            else:
                logger.info("体力不足，未设置使用理智药")
                self.no_potion = True
                return True
        elif scene == Scene.OPERATOR_RECOVER_ORIGINITE:
            logger.info("体力不足，没有理智药")
            self.no_potion = True
            return True
        elif scene == Scene.OPERATOR_SELECT:
            return True
        elif scene == Scene.OPERATOR_BEFORE:
            self.tap_element("ope_start", interval=2)
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            NavigationSolver().run(self.level_name, mode="copy")
